// escrow-address.js
// generate a 2-of-3 multi-signature Bitcoin address

var bitcoin = require('bitcoinjs-lib');
var prompt = require('cli-prompt');
var fs = require('fs');

// prompt for the 3 keys
console.log("Create a new escrow address from the public keys of each participant.");
prompt.multi([{ key: "buyer" },{ key: "seller" },{ key: "moderator" }], function (result) {

	// create a eliptic curve public key from the hex codes
	var pubKeys = [
		bitcoin.ECPubKey.fromHex(result.buyer),
		bitcoin.ECPubKey.fromHex(result.seller),
		bitcoin.ECPubKey.fromHex(result.moderator),
	]

	// generate a m-of-n multisignature output script
	var redeemScript = bitcoin.scripts.multisigOutput(2, pubKeys);
	var scriptPubKey = bitcoin.scripts.scriptHashOutput(redeemScript.getHash());
 
 	// generate the escrow bitcoin address and redeem script
 	// stored in a JSON file for organizing the subsequent steps
	var escrow_info = {
		address: bitcoin.Address.fromOutputScript(scriptPubKey).toString(),
		redeem_script: redeemScript.toHex().toString()
	}
 
 	console.log("\n");
	console.log("The address to your escrow:", escrow_info.address);

	// write the escrow info to disk for building a transaction
	var filename = "./escrow-" + escrow_info.address.substring(0,6) + ".json"
	fs.writeFile(filename, JSON.stringify(escrow_info, null, 4), function(err) {
    	if (err) {return console.log(err) }
    	console.log("Escrow information saved as:", filename);
	})
});
