// escrow-build-tx.js
// Create a transaction to withdraw the escrow balance

var bitcoin = require('bitcoinjs-lib');
var request = require('request');
var prompt = require('cli-prompt');
var fs = require('fs');

// fee to pay the miners
var tx_fee = 10000;	

// convert 'satoshi' to bitcoin value
var satoshiToBTC = function(value) {
	return value * 0.00000001;
}

// load the escrow json file
var escrow_filename = process.argv[2];
if (!escrow_filename) {
	console.log("Please provide the escrow filename.")
	console.log("example usage: node escrow-build-tx.js ./escrow-3Bh334.json")
	return;	
}
var escrow = require(escrow_filename);

console.log("Create a withdraw transaction for the escrow address:", escrow.address);
prompt('Choose a bitcoin address to send the balance: ', function (address) {	

	// query blockchain.info for the unspent outputs from the escrow address
	var url = "https://blockchain.info/unspent?active=" + escrow.address;
	request(url, function (error, response, body) {
  	if (!error && response.statusCode == 200) {

  		// parse the response and get the first unpsent output
  		var json = JSON.parse(body);
  		var unspent = json["unspent_outputs"][0];

  		// calculate the withdraw amount minus the tx fee
			var withdraw_amount = unspent.value - tx_fee;

  		// build a transaction 
			var txb = new bitcoin.TransactionBuilder(bitcoin.networks.bitcoin);
			var redeem_script = bitcoin.Script.fromHex(escrow.redeem_script);
			txb.addInput(unspent.tx_hash_big_endian, unspent.tx_output_n, bitcoin.Transaction.DEFAULT_SEQUENCE, redeem_script);
			txb.addOutput(address, withdraw_amount);
			var tx = txb.buildIncomplete();

			console.log("Unsigned transaction created to withdraw", 
				satoshiToBTC(withdraw_amount), "BTC with", satoshiToBTC(tx_fee), "BTC mining fee.");

			// save the unsigned transaction as json to disk
			escrow.signatures_required = 2;
			escrow.signatures = 0;
			escrow.pay_to_address = address;
			escrow.amount = withdraw_amount;
			escrow.fee = tx_fee;
			escrow.encoded_tx = tx.toHex();

			// write the json to disk
      fs.writeFile(escrow_filename, JSON.stringify(escrow, null, 4), function(err) {
  			if (err) { return console.log(err) }
  			console.log(escrow_filename, "has been updated with the withdraw transaction.")

			});

  	} else {
  		console.log("Unable to get an unspent transaction output.");
  		if (error)
  			console.log("ERROR:", error);
  	}
	});
})

