// escrow-key.js
// generates a random private/public key and address

var bitcoin = require('bitcoinjs-lib');
var fs = require('fs');

// create a new random key
var privKey = bitcoin.ECKey.makeRandom();

// save it in a JSON object to be written to disk
var key_info = {
	private_key: privKey.toWIF(),
	public_key: privKey.pub.toHex(),
	address: privKey.pub.getAddress().toString()
}

// display the key details to the console
console.log("A new private key has been generated for you\n");
console.log("private key:", key_info.private_key);
console.log("public key:", key_info.public_key);
console.log("public address:", key_info.address);
console.log("\n");

// write the key info to disk for backup 
var priv_file = "./key-private-" + key_info.address.substring(0,6) + ".json";
fs.writeFile(priv_file, JSON.stringify(key_info, null, 4), function(err) {
	if (err) { return console.log(err) }
	console.log("Private key saved as:", priv_file);
})

// write public key and address to disk for sharing
delete key_info["private_key"];  // hide private key
var pub_file = "./key-public-" + key_info.address.substring(0,6) + ".json";
fs.writeFile(pub_file, JSON.stringify(key_info, null, 4), function(err) {
	if (err) {return console.log(err) }
	console.log("Public key saved as:", pub_file);
})